"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("fs-extra");
const path = require("path");
const utils_1 = require("./utils");
const camel_case_1 = require("camel-case");
class ErudaWebpackPlugin {
    constructor(options = {}) {
        const isProdEnv = process.env.NODE_ENV === 'production';
        const { enable = !isProdEnv, plugins = [], container = null, tool = null, autoScale = true, useShadowDom = true, defaults, apiVersion = '', } = options;
        let { exclude = [] } = options;
        let isCli5 = false;
        if ((0, utils_1.isString)(exclude)) {
            exclude = [exclude];
        }
        else if ((0, utils_1.isRegExp)(exclude)) {
            exclude = [exclude];
        }
        else if (!Array.isArray(exclude)) {
            utils_1.log.warn(`\n[vue-cli-plugin-eruda] Invalid options: "exclude" must be regexp/regexp[] \n`);
            exclude = [];
        }
        if (apiVersion) {
            if (Number(apiVersion.split('.')[0]) >= 5) {
                isCli5 = true;
            }
        }
        this.options = {
            // plugin options
            enable,
            plugins,
            exclude,
            // eruda init options
            container,
            tool,
            autoScale,
            useShadowDom,
            defaults,
            apiVersion,
            isCli5,
        };
        // enable by default when NODE_ENV !== 'production'
        if (this.options.enable) {
            this._writeErudaEntry();
        }
    }
    // https://github.com/webpack/webpack/issues/6516
    _amendEntryWebpack(entry) {
        // vue-cli does not allow function type
        if ((0, utils_1.isEntryFunc)(entry))
            return (...args) => Promise.resolve(entry(...args)).then(this._amendEntryWebpack.bind(this));
        // webpack4 string | string[]
        if ((0, utils_1.isEntryItem)(entry)) {
            return this._injectErudaEntryPath(entry);
        }
        if ((0, utils_1.isPlainObject)(entry)) {
            const newEntryObject = {};
            Object.entries(entry).forEach(([key, val]) => {
                // webpack 4
                if ((0, utils_1.isEntryItem)(val)) {
                    newEntryObject[key] = this._injectErudaEntryPath(val);
                }
                // webpack 5
                if ((0, utils_1.isEntryDescription)(val)) {
                    newEntryObject[key] = Object.assign(Object.assign({}, val), { import: this._injectErudaEntryPath(val.import) });
                }
            });
            return newEntryObject;
        }
        return entry;
    }
    _injectErudaEntryPath(entry) {
        if ((0, utils_1.isString)(entry)) {
            return this._inExclude([entry]) ? [entry] : [this.erudaEntryPath, entry];
        }
        else if (Array.isArray(entry)) {
            return this._inExclude(entry) ? entry : [this.erudaEntryPath, ...entry];
        }
        return entry;
    }
    _inExclude(entry) {
        const { exclude = [] } = this.options;
        for (let i = 0; i < exclude.length; i++) {
            const rule = exclude[i];
            for (let j = 0; j < entry.length; j++) {
                const path = entry[j];
                if (((0, utils_1.isString)(rule) && path.startsWith(rule)) ||
                    ((0, utils_1.isRegExp)(rule) && rule.test(path))) {
                    return true;
                }
            }
        }
        return false;
    }
    _writeErudaEntry() {
        const entryPath = path.resolve(__dirname, './eruda-entry.js');
        const pluginStr = this._getPlugin();
        const optionsStr = this._getInitOptions();
        const erudaStr = `var eruda = require("eruda");window.eruda === undefined && (window.eruda = eruda);`;
        const initStr = `eruda.init(${optionsStr});`;
        const fileStr = erudaStr + initStr + pluginStr;
        const cache = fs.existsSync(entryPath)
            ? fs.readFileSync(entryPath).toString()
            : '';
        if (cache !== fileStr) {
            fs.writeFileSync(entryPath, fileStr, {
                encoding: 'utf8',
                flag: 'w',
            });
        }
        this.erudaEntryPath = require.resolve('./eruda-entry.js');
    }
    _getInitOptions() {
        const { container, tool, autoScale, useShadowDom, defaults } = this.options;
        const option = {
            container,
            autoScale,
            useShadowDom,
            defaults,
        };
        if (tool)
            option.tool = tool;
        if (defaults)
            option.defaults = defaults;
        return JSON.stringify(option);
    }
    _getPlugin() {
        const plugins = this.options.plugins;
        let pluginsStr = '';
        if (Array.isArray(plugins)) {
            [...plugins].forEach((val) => {
                let pluginName = val;
                let hasModule = false;
                if (!(0, utils_1.isString)(val))
                    return;
                if (!val.startsWith('eruda'))
                    pluginName = `eruda-${val}`;
                try {
                    require.resolve(pluginName);
                    hasModule = true;
                }
                catch (err) {
                    utils_1.log.warn(`\n[vue-cli-plugin-eruda] Error: Cannot find eruda plugin "${val}". \nYou may need to install it.\n`);
                }
                if (!hasModule)
                    return;
                const pluginCamelCase = (0, camel_case_1.camelCase)(pluginName);
                pluginsStr += `var ${pluginCamelCase} = require("${pluginName}");eruda.add(${pluginCamelCase});`;
            });
        }
        return pluginsStr;
    }
    apply(compiler) {
        if (this.options.enable) {
            compiler.options.entry = this._amendEntryWebpack(compiler.options.entry);
        }
    }
}
exports.default = ErudaWebpackPlugin;
